import { describe, it, expect, afterEach } from '@jest/globals'
import { build, createServer } from 'vite'

import vitePluginOxManifests from '../../dist/index'
import { getFileFromBundle, getPort, getDirname } from '../util'

const PORT = getPort()
const __dirname = getDirname(import.meta.url)

describe('Simple manifest scenario', function () {
  let server

  afterEach(async function () {
    if (server) {
      await server.close()
      server = null
    }
  })

  it('works', async function () {
    const bundle = await build({
      root: __dirname,
      logLevel: 'silent',
      build: { write: false, rollupOptions: { input: {} } },
      plugins: [vitePluginOxManifests({

      })]
    })

    const manifestFile = getFileFromBundle('manifest', bundle)
    const manifests = JSON.parse(manifestFile.source)
    expect(Object.keys(manifests)).toHaveLength(1)
    expect(manifests['register.js'].meta.manifests[0].namespace).toEqual('test')
  })

  it('works in dev mode', async function () {
    server = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginOxManifests({

      })]
    })
    await server.listen(PORT)

    const res = await fetch(`http://localhost:${PORT}/api/manifest.json`)
    const manifests = await res.json()

    expect(Object.keys(manifests)).toHaveLength(1)
    expect(manifests[0].namespace).toEqual('test')
    expect(manifests[0].path).toContain('register')
  })

  it('works in dev mode with base set', async function () {
    server = await createServer({
      root: __dirname,
      base: '/appsuite/',
      logLevel: 'silent',
      plugins: [vitePluginOxManifests({

      })]
    })
    await server.listen(PORT)

    const res = await fetch(`http://localhost:${PORT}/appsuite/api/manifest.json`)
    const manifests = await res.json()

    expect(Object.keys(manifests)).toHaveLength(1)
    expect(manifests[0].namespace).toEqual('test')
    expect(manifests[0].path).toContain('register')
  })

  it('works in dev mode with base set (new endpoint)', async function () {
    server = await createServer({
      root: __dirname,
      base: '/appsuite/',
      logLevel: 'silent',
      plugins: [vitePluginOxManifests({

      })]
    })
    await server.listen(PORT)

    const res = await fetch(`http://localhost:${PORT}/appsuite/manifests`)
    const manifests = await res.json()

    expect(Object.keys(manifests)).toHaveLength(1)
    expect(manifests[0].namespace).toEqual('test')
    expect(manifests[0].path).toContain('register')
  })

  it('does not empty files on transform', async function () {
    server = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginOxManifests({

      })]
    })
    await server.listen(PORT)

    const res = await fetch(`http://localhost:${PORT}/register.js`)
    expect(await res.text()).toEqual(expect.stringContaining("console.log('Hello world')\n"))
  })
})
